package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.SupplierInformationDTO;
import digital.laboratory.platform.reagent.entity.SupplierInformation;
import digital.laboratory.platform.reagent.service.SupplierInformationService;
import digital.laboratory.platform.reagent.vo.SupplierInformationVO;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;


import javax.activation.MimetypesFileTypeMap;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.Principal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * (服务商/供应商信息)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (服务商 / 供应商信息) 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/supplier_information")
@Api(value = "supplier_information", tags = "(服务商/供应商信息)管理")
public class SupplierInformationController {

    private final SupplierInformationService supplierInformationService;

    private final OssFile ossFile;

    /**
     * 通过id查询(服务商/供应商信息)
     *
     * @param supplierInformationId id
     * @return R
     */
    @ApiOperation(value = "通过id查询(服务商/供应商信息)", notes = "通过id查询(服务商/供应商信息)")
    @GetMapping()
	@PreAuthorize("@pms.hasPermission('reagent_supplier_information_get')" )
    public R<SupplierInformationVO> getById(String supplierInformationId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SupplierInformationVO voById = supplierInformationService.getVOById(supplierInformationId);

        return R.ok(voById);
    }

    /**
     * 分页查询
     *
     * @param page                分页对象
     * @param supplierName (服务商/供应商信息)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('reagent_supplier_information_page')" )
    public R<IPage<SupplierInformation>> getSupplierInformationPage(Page<SupplierInformation> page,  String supplierName, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();


        IPage<SupplierInformation> supplierInformationSList = supplierInformationService.page(page, Wrappers.<SupplierInformation>query()
                        .like(StrUtil.isNotBlank(supplierName),"supplier_name",supplierName)
                .orderByDesc("create_time")
        );
        return R.ok(supplierInformationSList);
//		return R.ok(supplierInformationService.page(page, Wrappers.query(supplierInformation)));
    }


    /**
     * 新增(服务商/供应商信息)
     *
     * @param supplierInformationDTO (服务商/供应商信息)
     * @return R
     */
    @ApiOperation(value = "新增服务商/供应商信息", notes = "新增服务商/供应商信息")
    @SysLog("新增服务商/供应商信息")
    @PostMapping
    @Transactional
    @PreAuthorize("@pms.hasPermission('reagent_supplier_information_add')" )
    public R<SupplierInformation> postAddObject(@RequestBody SupplierInformationDTO supplierInformationDTO, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SupplierInformation supplierInformation = supplierInformationService.addInfoById(supplierInformationDTO);

        if (supplierInformation != null) {
            return R.ok(supplierInformation, "保存成功");
        } else {
            return R.failed("保存失败");
        }
    }


    /**
     * 修改(服务商/供应商信息)
     *
     * @param supplierInformationDTO (服务商/供应商信息)
     * @return R
     */
    @ApiOperation(value = "修改服务商/供应商信息", notes = "修改服务商/供应商信息")
    @SysLog("修改服务商/供应商信息")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_supplier_information_edit')" )
    public R<SupplierInformation> putUpdateById(@RequestBody SupplierInformationDTO supplierInformationDTO, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SupplierInformation supplierInformation = supplierInformationService.editInfoById(supplierInformationDTO);

        if (supplierInformation != null) {
            return R.ok(supplierInformation, "修改成功");
        } else {
            return R.failed("修改失败");
        }
    }



    @ApiOperation(value = "上传资质文件", notes = "上传资质文件")
    @SysLog("上传资质文件")
    @PostMapping("/uploadQd")
    @PreAuthorize("@pms.hasPermission('reagent_supplier_information_add')" )
    public R<SupplierInformation> uploadById(String id, @RequestPart(value = "file", required = false) MultipartFile file, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SupplierInformation supplierInformation = supplierInformationService.getById(id);

        String path = "reagen_managment" + "/" + "supplierInformation/" + supplierInformation.getId();
        String fileName = FileNameUtil.getName(file.getOriginalFilename());
        boolean r = ossFile.fileUpload(file, path);
        Map<String, String> ResultData = new HashMap<>();
        ResultData.put("fileName", fileName);
        ResultData.put("path", path);

        if (r) {
            //修改数据库
            supplierInformation.setQualificationDocument(path);
            supplierInformation.setQualificationDocumentName(fileName);
            supplierInformationService.updateById(supplierInformation);
            String fileNames = path + "/" + fileName;
            ossFile.fileGet(fileNames, httpServletResponse.getOutputStream());
            return R.ok(supplierInformation, "上传成功");
        } else return R.failed("上传失败");
    }

    @ApiOperation(value = "预览资质文件", notes = "预览资质文件")
    @SysLog("预览资质文件")
    @GetMapping("/downloadQd")
    @PreAuthorize("@pms.hasPermission('reagent_supplier_information_get')" )
    public R<SupplierInformation> downloadById(String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SupplierInformation supplierInformation = supplierInformationService.getById(id);

        String qualificationDocumentName = supplierInformation.getQualificationDocumentName();

        if (qualificationDocumentName != null) {

            String path = "reagen_managment" + "/" + "supplierInformation/" + supplierInformation.getId();

            String fileName = path + "/" + qualificationDocumentName;

            ossFile.fileGet(fileName, httpServletResponse.getOutputStream());

            return R.ok(supplierInformation);

        } else return R.failed();


    }

    @ApiOperation(value = "预览供应人照片", notes = "预览供应人照片")
    @SysLog("预览供应人照片")
    @GetMapping("/download")
    @PreAuthorize("@pms.hasPermission('reagent_supplier_information_get')" )
    public R<SupplierInformation> download(String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SupplierInformation supplierInformation = supplierInformationService.getById(id);

        String photograph = supplierInformation.getPhotograph();

        if (!StrUtil.isNotBlank(photograph)){

            return R.failed(null);
        }

        if (photograph != null) {

            String path = "reagen_managment" + "/" + "supplierInformation/" + supplierInformation.getId();

            String fileName = path + "/" + photograph;

            ossFile.fileGet(fileName, httpServletResponse.getOutputStream());

            return R.ok(supplierInformation);

        } else return R.failed();
    }

    /**
     * 录入样品时上传图片，使用fastjson 解析json 对象
     *
     * @param id
     * @param jsonParam
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "上传供应商图片，使用fastjson 解析json 对象",
            notes = "上传供应商图片，使用fastjson 解析json 对象")
    @SysLog("上传供应商图片，使用fastjson 解析json 对象")
    @PostMapping(value = "/picture_base64/{id}")
    @PreAuthorize("@pms.hasAnyPermission('reagent_supplier_information_add')")
    public R uploadPictureBase64(@PathVariable("id") String id, @RequestBody JSONObject jsonParam) throws Exception {
//		System.out.println(String.format("uploadPhotoObj: hairSampleId=%s jsonParam.size()=%d", hairSampleId, jsonParam.size()));
        SupplierInformation byId = supplierInformationService.getById(id);
        if (byId != null) {
            String image = jsonParam.getStr("image");
            //System.out.println(String.format("image=%s", image));

            if (StringUtils.isNotEmpty(image)) {
                try {
                    String suffix = image.substring(11, image.indexOf(";"));
                    String fileName = DateUtil.format(new Date(), "yyyyMMddHHmmss") + "." + suffix;
                    System.out.println(String.format("fileName=[%s]", fileName));

                    //去掉头信息
                    String imgBase64 = image.substring(image.indexOf(",") + 1);
                    byte[] decoded = org.apache.commons.codec.binary.Base64.decodeBase64(imgBase64);
                    InputStream is = new ByteArrayInputStream(decoded);

                    // 生成存储路径
                    String path = "reagen_managment" + "/" + "supplierInformation" + "/" + byId.getId();


                    Map<String, String> ResultData = new HashMap<>();
                    ResultData.put("fileName", fileName);
                    ResultData.put("path", path);

                    boolean r = ossFile.fileSave(path + "/" + fileName, is);
                    if (r) {

                        byId.setPhotograph(fileName);
                        byId.setPhotographOfSupplier(path);
                        supplierInformationService.updateById(byId);
                        return R.ok(ResultData, "上传成功");

                    }
                    return R.failed("上传失败");

                } catch (Exception e) {
                    e.printStackTrace();
                    return R.failed("上传失败");
                }
            } else {
                return R.failed("上传的数据中没有图像");
            }
        }
        return R.failed("不存在这个供应商");
    }
}
